// BasicSumAgent.java

import swarm.Globals;
import swarm.defobj.Zone;
import swarm.objectbase.SwarmObjectImpl;

import swarm.objectbase.SwarmImpl;

import swarm.objectbase.EmptyProbeMap;
import swarm.objectbase.EmptyProbeMapImpl;

import java.util.*;

/**
 * This is the basic class of agents, that is inherited from all the others.
 * It contains the common caracteristics of the agents.
 * 
 *
 * @author Marco Agagliate, Antonio de Ruvo 
 *  
 */
public class BasicSumAgent extends SwarmObjectImpl
        {
        /** This is the number of the agent. */
        public int number;
        /** The quantity of orders sending out by the agent.
          * There is one share per order:
          * the quantity of the order is the number of orders.
          */
        public int iMax;
        /** If printing=1 many objects print data on the terminal
          * window; If printing=4 BasicSumAgents print data on
          * the terminal window.
          */
   
        public int printing;
        /** The price of the order. */
        public double price;
        /** The asimmetry of buy and sell probability. */
        public double asymmetricBuySellProb;
        /** The distribution of buy and sell order (fixed to 0.5). */
        public double buySellSwitch;
        /** The total agent's quantity of shares*/
        public double shareQuantity;
        /** The maximum number of order per agent. */
        public int maxOrderQuantity;
        /** The value of the shares at mean daily price. */
        public double shareValueAtMeanDailyPrice;
        /** The liquidity of the agent. */
        public double liquidityQuantity;
        /** The agent's wealth at mean daily price. */
        public double agentWealthAtMeanDailyPrice;
        /** The average price of the satified order. */
        public double meanOperatingPrice;
        /** The matrix of orders' prices. */
        public ArrayList executedPrices;
        /** The book of the model. */
        public Book theBook;

        /** Constructor for a new BasicSumAgent. */
        public BasicSumAgent(Zone aZone, int maxOrderQuantity)
                {
                super(aZone);
                executedPrices = new ArrayList();
                theBook = new Book(getZone());
                shareQuantity=0;
                shareValueAtMeanDailyPrice=0;
                liquidityQuantity=0;
                agentWealthAtMeanDailyPrice=0;
                }

        /** To extract data from the orders received from agents. */
        public double extract( ArrayList nameList, int indexOrder, int positionDatum ){
                double a=((double []) nameList.get(indexOrder))[positionDatum];
                return(a);
        }

        /** This method sets the number of the agent. */
        public void setNumber(int n)
                {
                number = n;
                System.out.println("the number of the agent is: "+number);
                }

        /** This method sets the asymmetricBuySellProb. */
        public void setAsymmetricBuySellProb(double p)
                {
                asymmetricBuySellProb=p;
                }

        /** This method sets the book. */
        public void setBook(Book b)
                {
                theBook=b;
                }

        /** This method sets the maximum number of order per agent. */
        public void setMaxOrderQuantity(int m)
                {
                maxOrderQuantity=m;
                System.out.println("la maxOrderQuantity e': "+maxOrderQuantity);
                }

        /** Option about the agent print capability. */
        public void setPrinting(int p)
                {
                printing=p;
                System.out.println("il valore di print e': "+printing);
                }

        /** The number of executed prices is increased */
        public void setConfirmationOfExecutedPrice(double[] p)
                {
                if(p[0]!=0 || p[1]!=0) 
                  executedPrices.add(p);
                }

        /**
         * This method exists only for the CurrentAgent.
         * Its implementation is defined into specify agent class.
         */
        public void act0()
                {
                }

        /**
         * This method exists only for the CurrentAgent.
         * Its implementation is defined into specify agent class.
         */
        public void act1()
                {
                }

        /**
         * This is the method that the agent calls at the end of the day.
         * The agent makes daily accounting.
         */
        public void act2()
                {
                double mP, q, TodayNumberOfShares;
                int i;

                if (printing==4) 
                        for(i=0;i<executedPrices.size();i++)
                                System.out.println("I'm agent " + number
                                + " and the book told me: "
                                + extract(executedPrices, i, 0)+ " for "  
                                + extract(executedPrices, i, 1)+ " share(s).");
                mP=theBook.getMeanPrice();
                q=0;
                for (i=0;i<executedPrices.size();i++)
                        {
                        if (extract(executedPrices, i, 0) > 0) 
                           q+=extract(executedPrices, i, 1);
                        if (extract(executedPrices, i, 0) < 0)
                           q-=extract(executedPrices, i, 1);
                        }
                shareQuantity += q;
                shareValueAtMeanDailyPrice=shareQuantity*mP; 
                meanOperatingPrice=0;
                for (i=0;i<executedPrices.size();i++)
                        {
                        liquidityQuantity  -= extract(executedPrices, i, 0)*
                                              extract(executedPrices, i, 1); 
                        meanOperatingPrice +=Math.abs(extract(executedPrices, i, 0)*
                                                      extract(executedPrices, i, 1));
                        }
		TodayNumberOfShares=0;
	        if(executedPrices.size() != 0){
                   for (i=0;i<executedPrices.size();i++)
		       TodayNumberOfShares+=extract(executedPrices, i, 1);
                   meanOperatingPrice/=TodayNumberOfShares;
		}
                else
                        meanOperatingPrice=mP;
                if (printing==4)
                        System.out.println("I'm agent " +  number
                        + " and the meanOperatingPrice is: "
                        +  meanOperatingPrice);
		System.out.println();
                agentWealthAtMeanDailyPrice
                        =shareValueAtMeanDailyPrice+liquidityQuantity;
		executedPrices.clear();
                }

        /** Return agent's wealth at mean daily price. */
        public double getWealthAtMeanDailyPrice()
                {
                return agentWealthAtMeanDailyPrice;
                }
        }

