// Book.java      

import swarm.defobj.Zone;
import swarm.objectbase.SwarmObjectImpl;
import java.util.*;

import swarm.collections.*;

import swarm.Globals;
import swarm.objectbase.EmptyProbeMap;
import swarm.objectbase.EmptyProbeMapImpl; 

/**
 * This is the book.
 * The book works on the basis of two arrayList containing sell order in
 * increasing order or buy order in decreasing order. The orders are arrays which 
 * have in the first position the price, in the second the number of agent who places
 * the order and in the third the quantity.
 * If an order obtains an immediate matching, it is not filed 
 * 
 * @author Antonio de Ruvo, integrazioni da parte di Bruno Mencarelli
 * @author antderu@libero.it 
 */

public class Book extends SwarmObjectImpl{
  /** The index used for send message to agents. */
  public ListIndex indexAgentListIndex;
  /** If 1 many objects print data on the terminal window. */
  public int printing;
  /** The number of orders received from agents. */
  public int orderNumber;
  /** The number of share negotiated */
  public int count=0;
  /** The number of shares in buy side of book. */ 
  public int sharesInBuySide;
  /** The number of shares in sell side of book. */
  public int sharesInSellSide;
  /** The number of shares in buy side of book before opening. */ 
  public int sharesInBuySideBeforeOpening;
  /** The number of shares in sell side of book before opening. */
  public int sharesInSellSideBeforeOpening;
  /** The price of the last contract closing. */
  public double executedPrice=1;
  /** The mean price of yesterday. */ 
  public double meanPrice=1;
  /** The closing price of yesterday. */
  public double  previousClosingPrice=executedPrice;
  /** The addition of prices*quantity to calculate the mean price. */
  public double currentMeanPrice=0;
  /* The arrayList which contains the buy orders. */
  public ArrayList buyOrderStorehouse=new ArrayList();
  /* The arrayList which contains the sell orders. */
  public ArrayList sellOrderStorehouse=new ArrayList();

  /* AGGIUNTE DA PARTE MIA */

  /** The length of the mean price history. */
  public int meanPriceHistoryLength=1000;
  /** The length of the local price history. */
  public int localHistoryLength=1000;
  /** The vector containing the history of mean prices. */
  public double[] meanPriceHistory = new double[meanPriceHistoryLength];
  /** The vector containing the local price history. */
  public double[] localHistory = new double[localHistoryLength];
    
  
  /** Constructor for a new book. */
  public Book(Zone aZone){
    super(aZone);
    }

  /** To inform the agent of executed price. 
   * @see BasicSumAgent
   */
  private void message(double n, double p, double q){
    BasicSumAgent anAgent;
    indexAgentListIndex.setOffset((int)n-1);
    anAgent = (BasicSumAgent)indexAgentListIndex.get();
    double[]confirmationOfExecutedPrice={p,q};
    anAgent.setConfirmationOfExecutedPrice(confirmationOfExecutedPrice);         
  }

  /** To standardize the order. */
  public double[] standardizedOrder(double[] orderFromAgent){
    double []  order=new double [4];
    if(orderFromAgent.length==2){
      double[] orderStandard={orderFromAgent[0], orderFromAgent[1], 1, 0};
      order=orderStandard;
    }
    if(orderFromAgent.length==3){
      double [] orderStandard={orderFromAgent[0], orderFromAgent[1],
                               orderFromAgent[2], 0};
      order=orderStandard;
    }
    if(orderFromAgent.length==4){
      double [] orderStandard={orderFromAgent[0], orderFromAgent[1],
                               orderFromAgent[2], orderFromAgent[3]};
      order=orderStandard;
    }
    return(order);      
  }

  /** To extract data from the orders received from agents. */
  public double extract( ArrayList nameList, int indexOrder, int positionDatum ){
    double a=((double []) nameList.get(indexOrder))[positionDatum];
    return(a);
  }

  /** To fill sellOrderStorehouse in increasing order */
  public int fillStorehouseIncreasingP(double newPrice){
    int index=0;
    for(int i=0; i<sellOrderStorehouse.size(); i++){
      if(newPrice<extract(sellOrderStorehouse, i, 0)){
	index=i;
	break;
      }
    }	
    return(index);    
  }

  /** To fill buyOrderStorehouse in descreasing order. */
  public int fillStorehouseDecreasingP (double newPrice){
    int index=0;
    for(int i=0; i<buyOrderStorehouse.size(); i++){
      if(newPrice>extract(buyOrderStorehouse, i, 0)){
	index=i;
	break;
      }
    }
    return(index);
  }

  // AGGIUNTE DA PARTE MIA

  /** This is the SetLocal method, used to shift the rows of the localHistory vector. */ 
  public void setLocal( double[] vect,double p ){
  int i;
  if(p==0) return;
  for(i=vect.length-2;i>=0;i--){
      vect[i+1]=vect[i];}
  vect[0]=p;
  }

  /** This is the getLocal method, that counts for the difference
   *  between the buying and selling orders sent by the agents.
   */
  public int getLocal( double[] vect ){
    int i,tot=0;
    for(i=0;i<vect.length;i++){
      if(vect[i]>0) tot++;
      if(vect[i]<0) tot--;
      }
  return tot;
  }

  /** To know the adress of memory of the agent. */
  public void setAgentListIndex(ListIndex i){
    indexAgentListIndex=i;
    }

  /** To set the value of option printing. */
  public void setPrinting(int p){
    printing=p;
  }

  /** At the end of each day to calculate the mean price. */
  public void setMeanPrice(){
    if(count>0)
      meanPrice=currentMeanPrice/count; 
    // otherwise we keep previous value
  }

  /** At the beginning of each day to clean the book. */
  public void setClean(){
    sellOrderStorehouse.clear();
    buyOrderStorehouse.clear();
    orderNumber=0;
    for(int i=meanPriceHistoryLength-2;i>=0;i--){
       meanPriceHistory[i+1]=meanPriceHistory[i];}
    meanPriceHistory[0]=meanPrice;
    previousClosingPrice=executedPrice; // the last of "yesterday"
    currentMeanPrice=0;
    count=0;
    sharesInSellSide=0;
    sharesInBuySide=0;
    if(printing==1)
	System.out.println("Book: clean");
  }

  /** Receiving an order before opening from an agent. */
  public void setOrderBeforeOpeningFromAgent(double[] orderFromAgent) {
    double [] order = standardizedOrder(orderFromAgent);
    if(printing==1)
      System.out.println("The book received a before opening order from agent " + order[1] +
      " at price " + order[0] + " for " + order[2]+ " share(s), with option = " + order[3]);
    orderNumber++;

  // AGGIUNTA DA PARTE MIA

  /* local History. */
  setLocal(localHistory,order[0]);

  /* if  price==0 or quantity==0 no action required, but sending a  message to the agent 
   * with price=0 */
    if(order[0]==0 || order[2]==0)
       message(order[1], 0.0, 0.0);

  // the agent is selling at minimum price if  price<0
     else if(order[0]<0) {
       message(order[1], 0.0, 0.0);
       order[0]=-order[0];
  /* to insert the order in the book in increasing order. 
   * With add( )the arrayList updates the index */
       if(sellOrderStorehouse.size()==0 ||
          order[0]>=extract(sellOrderStorehouse, (sellOrderStorehouse.size()-1), 0)) 
         sellOrderStorehouse.add(order);
       else
	 sellOrderStorehouse.add(fillStorehouseIncreasingP(order[0]), order);
       sharesInSellSide+=order[2];
       sharesInSellSideBeforeOpening=sharesInSellSide;
       if(printing==1) {
         for(int i =0; i<sellOrderStorehouse.size(); i++){
           for(int y=0; y<order.length; y++){
             System.out.print(extract(sellOrderStorehouse, i, y) + " ");
           }      
           System.out.println();
         };
       System.out.println();
       }
     }

  // the agent is buying at the maximun price if price>0
     else {
       message(order[1], 0.0, 0.0);
       /* to insert the order in the book in decreasing order. 
        * With add()the arrayList updates the index */
       if(buyOrderStorehouse.size()==0 ||
          order[0]<=extract(buyOrderStorehouse, (buyOrderStorehouse.size()-1) ,0)) 
         buyOrderStorehouse.add(order);
       else
	 buyOrderStorehouse.add(fillStorehouseDecreasingP(order[0]),order);
       sharesInBuySide+=order[2]; 
       sharesInBuySideBeforeOpening=sharesInBuySide;    
       if(printing==1) {
         for(int i=0; i<buyOrderStorehouse.size(); i++){
           for(int y=0; y<order.length; y++){
             System.out.print(extract(buyOrderStorehouse, i, y) + " ");
           }      
           System.out.println();
         };
       System.out.println();
       }
     }
  }

  // FASE DI NEGOZIAZIONE CONTINUA

  /** Receiving an order when the market is open. */
  public void setOrderFromAgent(double[] orderFromAgent) {
    double [] order = standardizedOrder(orderFromAgent);
    if(printing==1)
      System.out.println("The book received an order from agent " + order[1] +
      " at price " + order[0] + " for " + order[2]+ " share(s), with option = " + order[3]);
    orderNumber++;

  // AGGIUNTA DA PARTE MIA

  /* local History. */
  setLocal(localHistory,order[0]);

  /* if  price==0 or quantity==0 no action required, but sending a  message to the agent 
   * with price=0 */
    if(order[0]==0 || order[2]==0)
      message(order[1], 0.0, 0.0);

  // the agent is selling at minimum price if  price<0
     else if(order[0]<0) {
       /* if there is a compatible order, this one is removed and the order received  
        * it's not inserted in the book. It's not necessary to shift rows because
        * with remove() the arrayList updates the index */
       while(buyOrderStorehouse.size()>0 && extract(buyOrderStorehouse, 0, 0)>=-order[0]
             && order[2]!=0) {
	 if(order[2]==extract(buyOrderStorehouse, 0, 2)){ 
           count+=order[2];
           executedPrice=extract(buyOrderStorehouse, 0, 0);
           for(int i=0; i<order[2]; i++){
	     message(order[1], -executedPrice, order[2]);
	     message(extract(buyOrderStorehouse, 0, 1), executedPrice, order[2]);
	   }
	   currentMeanPrice+=executedPrice*order[2];
           buyOrderStorehouse.remove(0);
           sharesInBuySide-=order[2];
	   order[2]=0;
           if(printing==1) {
             for(int i=0; i<buyOrderStorehouse.size(); i++){
               for(int y=0; y<order.length; y++){
                 System.out.print(extract(buyOrderStorehouse, i, y) + " ");
               }      
               System.out.println();
             };
           System.out.println();
           }
	 }
	 else if(order[2]<extract(buyOrderStorehouse, 0, 2)){
           count+=order[2];
           executedPrice=extract(buyOrderStorehouse, 0, 0);
           for(int i=0; i<order[2]; i++){
	     message(order[1], -executedPrice, order[2]);
	     message(extract(buyOrderStorehouse, 0, 1), executedPrice, order[2]);
	   }
	   currentMeanPrice+=executedPrice*order[2];
	   ((double[]) buyOrderStorehouse.get(0))[2]=extract(buyOrderStorehouse, 0, 2)
                                                     -order[2];
           sharesInBuySide-=order[2];
	   order[2]=0;
           if(printing==1) {
             for(int i=0; i<buyOrderStorehouse.size(); i++){
               for(int y=0; y<order.length; y++){
                 System.out.print(extract(buyOrderStorehouse, i, y) + " ");
               }      
               System.out.println();
             };
           System.out.println();
           }
	 }
	 else if(order[2]>extract(buyOrderStorehouse, 0, 2)){
           count+=extract(buyOrderStorehouse, 0, 2);
           executedPrice=extract(buyOrderStorehouse, 0, 0);
           for(int i=0; i<extract(buyOrderStorehouse, 0, 2); i++){
	     message(order[1], -executedPrice, extract(buyOrderStorehouse, 0, 2));
	     message(extract(buyOrderStorehouse, 0, 1), executedPrice,
                     extract(buyOrderStorehouse, 0, 2));
	   }
	   currentMeanPrice+=executedPrice*extract(buyOrderStorehouse, 0, 2);
	   order[2]=order[2]-extract(buyOrderStorehouse, 0, 2);
           sharesInBuySide-=extract(buyOrderStorehouse, 0, 2);
           buyOrderStorehouse.remove(0);
           if(printing==1) {
             for(int i=0; i<buyOrderStorehouse.size(); i++){
               for(int y=0; y<order.length; y++){
                 System.out.print(extract(buyOrderStorehouse, i, y) + " ");
               }      
               System.out.println();
             };
           System.out.println();
           }
	 }	   
       }
       if(order[2]!=0) {
	 message(order[1], 0.0, 0.0);
         order[0]=-order[0];
  /* to insert the order in the book in increasing order. 
   * With add( )the arrayList updates the index */
         if(sellOrderStorehouse.size()==0 ||
            order[0]>=extract(sellOrderStorehouse, (sellOrderStorehouse.size()-1), 0)) 
           sellOrderStorehouse.add(order);
	 else
	   sellOrderStorehouse.add(fillStorehouseIncreasingP(order[0]), order); 
         sharesInSellSide+=order[2];
         if(printing==1) {
           for(int i =0; i<sellOrderStorehouse.size(); i++){
             for(int y=0; y<order.length; y++){
               System.out.print(extract(sellOrderStorehouse, i, y) + " ");
             }      
             System.out.println();
           };
         System.out.println();
         }
       }
     }

  // the agent is buying at the maximun price if price>0
     else {
       /* if there is a compatible order, this one is removed and the order received  
        * it's inserted in the book. It's not necessary shift rows because
        * with remove() the arrayList updates the index */
       while(sellOrderStorehouse.size()>0 && extract(sellOrderStorehouse, 0, 0)<=order[0]
             && order[2]!=0) {
	 if(order[2]==extract(sellOrderStorehouse, 0, 2)){ 
           count+=order[2];
           executedPrice=extract(sellOrderStorehouse, 0, 0);
           for(int i=0; i<order[2]; i++){
	     message(order[1], executedPrice, order[2]);
	     message(extract(sellOrderStorehouse, 0, 1), -executedPrice, order[2]);
	   }
	   currentMeanPrice+=executedPrice*order[2];
           sellOrderStorehouse.remove(0);
           sharesInSellSide-=order[2];
           order[2]=0;
           if(printing==1) {
             for(int i=0; i<sellOrderStorehouse.size(); i++){
               for(int y=0; y<order.length; y++){
                 System.out.print(extract(sellOrderStorehouse, i, y) + " ");
               }      
               System.out.println();
             };
           System.out.println();
           }
	 }
	 else if(order[2]<extract(sellOrderStorehouse, 0, 2)){
           count+=order[2];
           executedPrice=extract(sellOrderStorehouse, 0, 0);
           for(int i=0; i<order[2]; i++){
	     message(order[1], executedPrice, order[2]);
	     message(extract(sellOrderStorehouse, 0, 1), -executedPrice, order[2]);
	   }
	   currentMeanPrice+=executedPrice*order[2];
	   ((double[]) sellOrderStorehouse.get(0))[2]=extract(sellOrderStorehouse, 0, 2)
                                                      -order[2];
           sharesInSellSide-=order[2];
	   order[2]=0;
           if(printing==1) {
             for(int i=0; i<sellOrderStorehouse.size(); i++){
               for(int y=0; y<order.length; y++){
                 System.out.print(extract(sellOrderStorehouse, i, y) + " ");
               }      
               System.out.println();
             };
           System.out.println();
           }
	 }
	 else if(order[2]>extract(sellOrderStorehouse, 0, 2)){
           count+=extract(sellOrderStorehouse, 0, 2);
           executedPrice=extract(sellOrderStorehouse, 0, 0);
           for(int i=0; i<extract(sellOrderStorehouse, 0, 2); i++){
	     message(order[1], executedPrice, extract(sellOrderStorehouse, 0, 2));
	     message(extract(sellOrderStorehouse, 0, 1), -executedPrice,
                     extract(sellOrderStorehouse, 0, 2));
	   }
	   currentMeanPrice+=executedPrice*extract(sellOrderStorehouse, 0, 2);
	   order[2]=order[2]-extract(sellOrderStorehouse, 0, 2);
           sharesInSellSide-=extract(sellOrderStorehouse, 0, 2);
           sellOrderStorehouse.remove(0);
           if(printing==1) {
             for(int i=0; i<sellOrderStorehouse.size(); i++){
               for(int y=0; y<order.length; y++){
                 System.out.print(extract(sellOrderStorehouse, i, y) + " ");
               }      
               System.out.println();
             };
           System.out.println();
           }
	 }
       }
       if(order[2]!=0) {
	 message(order[1], 0.0, 0.0);
  /* to insert the order in the book in decreasing order. 
   * With add()the arrayList updates the index */
         if(buyOrderStorehouse.size()==0 ||
            order[0]<=extract(buyOrderStorehouse, (buyOrderStorehouse.size()-1) ,0)) 
           buyOrderStorehouse.add(order);
	 else
	   buyOrderStorehouse.add(fillStorehouseDecreasingP(order[0]),order);
         sharesInBuySide+=order[2];     
         if(printing==1) {
           for(int i=0; i<buyOrderStorehouse.size(); i++){
             for(int y=0; y<order.length; y++){
               System.out.print(extract(buyOrderStorehouse, i, y) + " ");
             }      
             System.out.println();
           };
         System.out.println();
         }
       }
     }
  }

  /** To get the last executed price. */
  public double getPrice(){
    return(executedPrice);
  }

  /** To get the mean price of yesterday. */
  public double getMeanPrice(){
    return(meanPrice);
  }

  // AGGIUNTE DA PARTE MIA

  /** to get the lagged Mean Price. */
  public double getLaggedMeanPrice(int lag){
    return meanPriceHistory[lag-1];
  }

  /** to get the local History. */
  public int getLocalHistory(){
    return getLocal(localHistory);
  }

  /** To get the number of shares filled in the sell side of book. */
  public int getSharesInSellSide(){
    return(sharesInSellSide);
  }

  /** To get the number of shares filled in the buy side of book. */
  public int getSharesInBuySide(){
    return(sharesInBuySide);
  }

  /** To get the closing price of yesterday. */
  public double getPreviousClosingPrice(){
    return(previousClosingPrice);
  }

  /** To get the spread between the first and the last price in the sell side. */
  public double getSellFirstLastSpread(){
    double spread=0;
    if(sellOrderStorehouse.size()==0)
      spread=0;
    else
      spread=extract(sellOrderStorehouse, 0, 0)-
	  extract(sellOrderStorehouse, sellOrderStorehouse.size()-1, 0);
    return(spread);
  }

  /** To get the spread between the first and the last price in the buy side. */
  public double getBuyFirstLastSpread(){
    double spread=0;
    if(buyOrderStorehouse.size()==0)
      spread=0;
    else
      spread=extract(buyOrderStorehouse, 0, 0)-
	  extract(buyOrderStorehouse, buyOrderStorehouse.size()-1, 0);
    return(spread);
  }

  /** To get the spread between first bid and first ask price. */
  public double getSpread(){
    double spread=0;
    if(sellOrderStorehouse.size()==0 && buyOrderStorehouse.size()>0)
      spread=0-extract(buyOrderStorehouse, 0, 0);
    else if(sellOrderStorehouse.size()==0 && buyOrderStorehouse.size()==0)
      spread=0;
    else if(sellOrderStorehouse.size()>0 && buyOrderStorehouse.size()==0)
      spread=extract(sellOrderStorehouse, 0, 0)-0;
    else
    spread=extract(sellOrderStorehouse, 0, 0)-extract(buyOrderStorehouse, 0, 0);
    return(spread);
  }

  /** To get the spread between quantities in sell and buy side. */
  public int getBuySellQuantitySpread(){
    return(sharesInBuySide-sharesInSellSide);
  }

  /** To get the number of shares in sell side before opening. */
  public int getSharesInSellSideBeforeOpening(){
    return(sharesInSellSideBeforeOpening);
  }

  /** To get the number of shares in buy side before opening. */
  public int getSharesInBuySideBeforeOpening(){
    return(sharesInBuySideBeforeOpening);
  }

}
