// locallyImitatingAgent.java

import swarm.Globals;
import swarm.defobj.Zone;
import swarm.objectbase.SwarmObjectImpl;

/**
 *LocallyImitatingAgent inherits from BasicSumAgent and operates in an imitative way,
 *following other agents actions(last operating agents or "local" imitation).
 *
 *@author Bruno Mencarelli
 *
 */

public class LocallyImitatingAgent extends BasicSumAgent
        {
        /** 
         *This is the vector containing the basic input of an order:
         *the price of the offer, the quantity of shares and the identification number of the agent.
         *
         */ 
        public double[] order = new double[3];

        /**This is the RuleMaster of the locally imitating agent.*/
        RandomRuleMaster ruleMaster;

        /** Constructor for a new locally imitating agent. */
        public LocallyImitatingAgent(Zone aZone, int maxOrderQuantity)
                {
                super(aZone, maxOrderQuantity);
                }

        /** This method allows the agent to operate following the other agents' actions. */ 
        public double chooseBuySellSwitch(Book theBook,double asymmetricBuySellProb)
                {
                double buySellSwitch;
                buySellSwitch=0.5; // la situazione standard di scelta casuale
                //Imitating other agents
                if(theBook.getLocalHistory()>0){
                buySellSwitch=asymmetricBuySellProb;
                System.out.println("I'm a locallyImitating agent and I'm acting at a prob: "+buySellSwitch);
                System.out.println("because the local History is: "+theBook.getLocalHistory());
                }
                else if(theBook.getLocalHistory()<0){
                buySellSwitch=1-asymmetricBuySellProb;
                System.out.println("I'm a locallyImitating Agent and I'm acting at a prob: "+buySellSwitch);
                System.out.println("because the local History is: "+theBook.getLocalHistory());
                }
                return buySellSwitch;
                } 
 
        /** This method sets the Rule Master. */
        public void setRuleMaster(RandomRuleMaster r)
                {
                ruleMaster=r;
                }

        /** This method is called in the first phase of the day. */
        public void act0()
                {
                double lastPrice;
                int i;

                //Imitating other agents...
                buySellSwitch=this.chooseBuySellSwitch(theBook,asymmetricBuySellProb);
                lastPrice=theBook.getPrice(); 
                price = ruleMaster.getPriceBeforeOpening (lastPrice, buySellSwitch);
                if (maxOrderQuantity==1) 
                        iMax=1;
                else
                        iMax=Globals.env.uniformIntRand.getIntegerWithMin$withMax(1, maxOrderQuantity); 

                order[0]=price;
                order[1]=number;
                order[2]=iMax;               
                theBook.setOrderBeforeOpeningFromAgent (order);
                System.out.println("I'm the locallyImitatingAgent number "+order[1]+" in act0 and I sent an order for "+order[2]+" shares at price "+order[0]);
                }

        /**
         *This method is called during the simulated day. 
         *The agent acts in the market.
         */
        public void act1()
                {
                double lastPrice;
                int i;
                //imitating other agents...
                buySellSwitch=this.chooseBuySellSwitch(theBook,asymmetricBuySellProb);
                lastPrice=theBook.getPrice();
                price = ruleMaster.getPrice (lastPrice, buySellSwitch);
                if (maxOrderQuantity==1) 
                   iMax=1;
                else
                   iMax=Globals.env.uniformIntRand.getIntegerWithMin$withMax(1, maxOrderQuantity); 
              
                order[0]=price;
                order[1]=number;
                order[2]=iMax; 
                theBook.setOrderFromAgent (order);
                System.out.println("I'm the locallyImitatingAgent number "+order[1]+" in act1 and I sent an order for "+order[2]+" shares at price "+order[0]);
                }
        }



