// marketImitatingAgent.java

import swarm.Globals;
import swarm.defobj.Zone;
import swarm.objectbase.SwarmObjectImpl;

/**
 *MarketImitatingAgent inherits from BasicSumAgent and operates in an imitative way,
 *following market tendency.
 *
 *@author Bruno Mencarelli
 *
 */

public class MarketImitatingAgent extends BasicSumAgent
        {
        /** 
         *This is the vector containing the basic input of an order:
         *the price of the offer, the quantity of shares and the identification number of the agent.
         *
         */  
        public double[] order = new double[3];

        /**This is the Rule Master of the market imitating agent. */
        public RandomRuleMaster ruleMaster;

        /** Constructor for a new market imitating agent. */
        public MarketImitatingAgent(Zone aZone, int maxOrderQuantity)
                {
                super(aZone, maxOrderQuantity);
                }

        /** This method allows the agent to operate following the market tendency. */
        public double chooseBuySellSwitch(Book theBook,double asymmetricBuySellProb)
                {
                double buySellSwitch,lagged1MeanPrice,lagged2MeanPrice;
                buySellSwitch=0.5; // la situazione standard di scelta casuale
                lagged1MeanPrice=theBook.getLaggedMeanPrice(1);//il prezzo medio in t-1
                lagged2MeanPrice=theBook.getLaggedMeanPrice(2);//il prezzo medio in t-2
                //imitating the market...
                if(lagged1MeanPrice>lagged2MeanPrice){
                   buySellSwitch=asymmetricBuySellProb;
                System.out.println("I'm a marketImitating agent and I'm acting at a prob: "+buySellSwitch);
                System.out.println("the lagged mean price at t=1 is "+lagged1MeanPrice+" and the lagged mean price at time t=2 is "+lagged2MeanPrice);                
                }
                else if(lagged1MeanPrice<lagged2MeanPrice){
                   buySellSwitch=1-asymmetricBuySellProb;
                System.out.println("I'm a marketImitating agent and I'm acting at a prob: "+buySellSwitch);
                System.out.println("the lagged mean price at t=1 is "+lagged1MeanPrice+" and the lagged mean price at time t=2 is "+lagged2MeanPrice);
                }
                return buySellSwitch;
                }
  
        /** This method sets the Rule Master. */
        public void setRuleMaster(RandomRuleMaster r)
                {
                ruleMaster=r;
                }

        /** This method is called in the first phase of the day. */ 
        public void act0()
                {
                double lastPrice;
                int i;

                //imitando il mercato...
                buySellSwitch=this.chooseBuySellSwitch(theBook,asymmetricBuySellProb);
                lastPrice=theBook.getPrice(); 
                price = ruleMaster.getPriceBeforeOpening (lastPrice, buySellSwitch);
                if (maxOrderQuantity==1) 
                        iMax=1;
                else
                        iMax=Globals.env.uniformIntRand.getIntegerWithMin$withMax(1, maxOrderQuantity);
                order[0]=price;
                order[1]=number;
                order[2]=iMax;
                theBook.setOrderBeforeOpeningFromAgent (order);
                System.out.println("I'm the marketImitatingAgent number "+order[1]+" in act0 and I sent an order for "+order[2]+" shares at price "+order[0]);
                }

        /**
         *This method is called during the simulated day.
         *The agent acts in the market.
         */
        public void act1()
                {
                double lastPrice;
                int i;

                //imitando il mercato...
                buySellSwitch=this.chooseBuySellSwitch(theBook,asymmetricBuySellProb);
                lastPrice=theBook.getPrice();
                price = ruleMaster.getPrice (lastPrice, buySellSwitch);
                if (maxOrderQuantity==1) 
                   iMax=1;
                else
                   iMax=Globals.env.uniformIntRand.getIntegerWithMin$withMax(1, maxOrderQuantity);
                order[0]=price;
                order[1]=number;
                order[2]=iMax;
                theBook.setOrderFromAgent (order);
                System.out.println("I'm the marketImitatingAgent number "+order[1]+" in act1 and I sent an order for "+order[2]+" shares at price "+order[0]);
                }
        }



